"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertBashToWin = void 0;
var tslib_1 = require("tslib");
// patch for irrelevant node dependency of bash-parser:
if (typeof window === 'undefined') {
    // @ts-ignore
    var window_1 = { process: { env: { NODE_NEV: 'mock' } } };
    // @ts-ignore
}
else if (!window.process) {
    // @ts-ignore
    window.process = { env: { NODE_NEV: 'mock' } };
}
var bash_parser_1 = tslib_1.__importDefault(require("bash-parser"));
var rm_handler_1 = require("./rm-handler");
var cp_handler_1 = require("./cp-handler");
function changePath(path) {
    return path
        .replace(/^\/(\w)\//g, '$1:\\') // cygwin windows paths
        .replace(/^\./, '%CD%')
        .replace(/\//g, '\\');
}
function performExpansions(text, expansions) {
    // currently assumes only ParameterExpansions (TODO CommandExpansion, ArithmeticExpansion)
    var result = text || '';
    var sortedExpansions = tslib_1.__spreadArray([], (expansions || []));
    sortedExpansions.sort(function (a, b) { return a.loc.start > b.loc.start ? -1 : 1; });
    for (var _i = 0, sortedExpansions_1 = sortedExpansions; _i < sortedExpansions_1.length; _i++) {
        var expansion = sortedExpansions_1[_i];
        // expand function parameters such as `$1` (-> `%~1`) different to regular variables `$MY`(-> `%MY%`):
        var expandedValue = /^\d+$/.test("" + expansion.parameter) ? "%~" + expansion.parameter : "%" + expansion.parameter + "%";
        result = "" + result.substring(0, expansion.loc.start) + expandedValue + result.substring(expansion.loc.end + 1);
    }
    return result;
}
var pathMatcher = /(\/?[.\w]+(?:[.\w]+\/)*\/?|\b\.\b)/ig;
var ConvertBash = /** @class */ (function () {
    function ConvertBash() {
        var _this = this;
        this.rmHandler = new rm_handler_1.RmHandler(function (c) { return _this.convertCommand(c); });
        this.cpHandler = new cp_handler_1.CpHandler(function (c) { return _this.convertCommand(c); });
        this.userDefinedFunctionNames = new Set();
        this.userDefinedFunctions = [];
    }
    ConvertBash.prototype.convertCommand = function (command) {
        var _this = this;
        console.log('convertCommand', command);
        var text = performExpansions(command.text, command.expansion)
            .replace(pathMatcher, function (match) { return changePath(match); });
        switch (command.type) {
            case 'Command':
                if (command.prefix && command.prefix.length && (!command.name || !command.name.text)) { // simple variable assignment
                    return command.prefix.map(function (c) { return _this.convertCommand(c); }).join('\n');
                }
                if (command.name && command.name.text) {
                    if (this.userDefinedFunctionNames.has(command.name.text)) {
                        var params = command.suffix && command.suffix.length ? ' ' + command.suffix.map(this.convertCommand).join(' , ') : '';
                        return "CALL :" + command.name.text + params;
                    }
                    var suffix = command.suffix ? " " + command.suffix.map(function (c) { return _this.convertCommand(c); }).join(' ') : '';
                    switch (command.name.text) {
                        case 'set':
                            if (command.suffix && command.suffix.length === 1 && command.suffix[0].text === '-e') {
                                console.log('skipping "set -e"');
                                return '';
                            }
                            else {
                                return "" + command.name.text + suffix;
                            }
                        case 'rm':
                            return this.rmHandler.handle(command);
                        case 'cp':
                            return this.cpHandler.handle(command);
                        default:
                            return "" + command.name.text + suffix;
                    }
                }
                return 'unknown command: ' + JSON.stringify(command);
            case 'Function':
                // NOTE: bash: definition before call. batch: call before definition
                // --> append function definitions at the end
                this.userDefinedFunctionNames.add(command.name.text);
                this.userDefinedFunctions.push(command);
                return '';
            case 'Word':
                if (text === '') {
                    return '""';
                }
                if (text && text.indexOf(' ') >= 0 && !text.startsWith('"')) {
                    return "\"" + text + "\"";
                }
                return text;
            case 'AssignmentWord':
                var _a = text.split('=', 2), variableName = _a[0], variableValue = _a[1];
                return "SET " + variableName + "=" + variableValue;
            case 'LogicalExpression':
                switch (command.op) {
                    case 'and':
                        return (this.convertCommand(command.left)) + " && " + (this.convertCommand(command.right));
                    case 'or':
                        return (this.convertCommand(command.left)) + " || " + (this.convertCommand(command.right));
                    default:
                        return "REM UNKNOWN operand \"" + command.op + "\" in: " + JSON.stringify(command);
                }
            case 'CompoundList':
                return command.commands.map(function (c) { return _this.convertCommand(c); }).join('\n');
            case 'If':
                // note: AND/OR is not supported with batch IF (https://stackoverflow.com/a/2143203/2544163)
                var condition = this.convertCommand(command.clause.commands[0]).replace(/^\[ | ]$/g, '');
                var elseBranch = command.else ? " ELSE (\n" + this.indent(this.convertCommand(command.else)) + "\n)" : '';
                return "IF " + condition + " (\n" + this.indent(this.convertCommand(command.then)) + "\n)" + elseBranch;
        }
        return 'REM UNKNOWN: ' + JSON.stringify(command);
    };
    ConvertBash.prototype.getFunctionDefinitions = function () {
        var _this = this;
        if (!this.userDefinedFunctions.length) {
            return '';
        }
        return "\n\nEXIT /B %ERRORLEVEL%\n\n" + this.userDefinedFunctions.map(function (f) {
            var innerCommands = f.body.commands.map(function (c) { return _this.convertCommand(c); }).join('\n');
            return ":" + f.name.text + "\n" + innerCommands + "\nEXIT /B 0\n";
        }).join('\n');
    };
    ConvertBash.prototype.indent = function (s) {
        return s.split('\n').map(function (line) { return "  " + line; }).join('\n');
    };
    return ConvertBash;
}());
function convertBashToWin(script) {
    var ast = bash_parser_1.default(script, { mode: 'bash' });
    var converter = new ConvertBash();
    return '@echo off\n\n' +
        ast.commands
            .map(function (c) { return converter.convertCommand(c); })
            .filter(function (c) { return !!c; }) // filter empty commands
            .join('\n') +
        converter.getFunctionDefinitions();
}
exports.convertBashToWin = convertBashToWin;
